import { isVNode, defineComponent, ref, getCurrentInstance, watch, computed, createVNode, resolveComponent, mergeProps } from 'vue';
import { logWarn } from '../../../utils/index.mjs';
import cellEdit from './cellEdit.mjs';
import { ElTable, ElTableColumn, ElRadio } from 'element-plus';
import { useSelection } from './useSelection.mjs';
import { getValue, setValue, extractObject } from '../../../utils/tools.mjs';

function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !isVNode(s);
}
var customTable = defineComponent({
  name: "CustomTable",
  components: {
    CellEdit: cellEdit,
    ElTable,
    ElTableColumn,
    ElRadio
  },
  props: {
    emptyText: {
      type: [String, Function],
      default: ""
    },
    rowSelection: {
      type: Object,
      default: () => ({
        selectedRowKeys: [],
        onChange: () => ({})
      })
    },
    rowKey: {
      type: [String, Function],
      default: void 0
    },
    disableTravel: Boolean,
    bodyBorder: Boolean,
    wrapHeader: Boolean,
    showOverflowTooltip: Boolean,
    hiddenCurrent: Boolean,
    rowEdit: Boolean,
    banSelectAll: Boolean,
    customIcon: Boolean,
    data: {
      type: Array,
      default: () => []
    },
    config: {
      type: Array,
      default: () => []
    },
    htmlContent: Boolean
  },
  emits: ["save", "radio", "select"],
  setup(props, context) {
    const tableConfig = ref([]);
    const radio = ref("");
    const instance = getCurrentInstance();
    const tableRef = ref();
    const rowSelection = ref();
    watch(() => props.rowSelection, () => {
      rowSelection.value = props.rowSelection ? {
        ...props.rowSelection
      } : props.rowSelection;
    }, {
      deep: true,
      immediate: true
    });
    const [renderSelect, renderSelectTop] = useSelection(rowSelection, {
      pageData: computed(() => props.data),
      getRowKey,
      getRecordByKey: getRowByKey
    });
    watch(() => props.config, (config) => {
      tableConfig.value = [...config];
    }, {
      immediate: true,
      deep: true
    });
    function clearSelection() {
      tableRef.value.clearSelection();
    }
    function getRowKey(row, rowKey = props.rowKey, index) {
      if (!rowKey) {
        console.warn("\u6CA1\u6709\u8BBE\u7F6Erow-key");
        return index;
      }
      if (typeof rowKey === "function") {
        return rowKey(row);
      } else if (row[rowKey]) {
        return row[rowKey];
      } else {
        return false;
      }
    }
    function getRowByKey(key) {
      return props.data.find((item) => getRowKey(item) === key);
    }
    const renderRadio = (row, index, config) => createVNode(resolveComponent("el-radio"), mergeProps({
      "model-value": radio.value
    }, {
      "onUpdate:modelValue": (val) => {
        radio.value = val;
        context.emit("radio", radio.value, row);
        context.emit("select", [row]);
      }
    }, {
      "label": getRowKey(row, props.rowKey, index),
      "disabled": config.selectable ? !config.selectable(row) : false
    }), {
      default: () => [createVNode("span", null, null)]
    });
    const renderCellEdit = (row, column, config) => createVNode(resolveComponent("cell-edit"), {
      "modelValue": getValue(row, column.property, config, props.disableTravel),
      "onBlur": (val) => {
        setValue(row, column.property, val);
      },
      "onSave": (cell) => {
        var _a;
        (_a = instance == null ? void 0 : instance.parent) == null ? void 0 : _a.emit("save", cell, column.property);
      }
    }, null);
    const getColumnSlot = (data, config) => {
      var _a, _b;
      if (config.children && config.children.length > 0) {
        return config.children.map((config2) => transformTableColumn(config2));
      }
      const {
        row,
        column,
        $index
      } = data;
      if (config.type === "expand") {
        return (_b = (_a = context.slots).expand) == null ? void 0 : _b.call(_a, data);
      } else if (config.type === "radio") {
        logWarn("[bc-table] radio\u5C06\u88AB\u5F03\u7528\uFF0C\u8BF7\u4F7F\u7528row-selection.type = 'radio', \u5177\u4F53\u8BF7\u53C2\u8003https://sepveneto.github.io/basic-comp/components/table.html#%E5%8D%95%E9%80%89");
        if (!~$index) {
          return;
        }
        return renderRadio(row, $index, config);
      } else if (config.editable) {
        return renderCellEdit(row, column, config);
      }
      if (!column.property) {
        return null;
      }
      const slot = context.slots[config.prop];
      const value = getValue(row, column.property, config, props.disableTravel);
      const realValue = typeof props.emptyText === "function" ? props.emptyText(value, column) : value === "" ? props.emptyText : value;
      return slot ? slot(data) : createVNode("span", null, [realValue]);
    };
    const transformTableColumn = (config) => {
      if (!config) {
        return null;
      }
      if (config.type === "select") {
        return createVNode(resolveComponent("el-table-column"), {
          "width": "48px"
        }, {
          default: (data) => renderSelect(data, config),
          header: () => renderSelectTop()
        });
      }
      if (config.type === "selection") {
        return createVNode(resolveComponent("el-table-column"), config, null);
      }
      return createVNode(resolveComponent("el-table-column"), mergeProps({
        "show-overflow-tooltip": props.showOverflowTooltip
      }, extractObject(config, ["children"], "exclude")), {
        default: (data) => getColumnSlot(data, config),
        header: ({
          column,
          $index
        }) => {
          const header = context.slots[`${config.prop}-header`];
          return header ? header({
            column,
            $index
          }) : createVNode("span", null, [column.label]);
        }
      });
    };
    context.expose({
      clearSelection,
      getRef: () => tableRef.value
    });
    return {
      tableRef,
      tableConfig,
      transformTableColumn
    };
  },
  render() {
    let _slot;
    return createVNode(resolveComponent("el-table"), mergeProps({
      "class": ["custom-table", {
        "cutom-icon": this.customIcon
      }, {
        "current-hover": this.hiddenCurrent
      }],
      "ref": "tableRef",
      "border": true,
      "data": this.data,
      "header-cell-class-name": "custom-header",
      "row-key": this.rowKey,
      "onSelect": (...args) => this.$emit("select", args),
      "size": "default"
    }, this.$attrs), _isSlot(_slot = this.tableConfig.map((config) => this.transformTableColumn(config))) ? _slot : {
      default: () => [_slot]
    });
  }
});

export { customTable as default };
//# sourceMappingURL=customTable.mjs.map
